'use strict';

var Component = require('../component');
var Element = require('../element');

var defaults = {
	prefix: 'ui',
	type: null, // push, file
	element: {
		tag: 'button'
	},
	ink: {
		duration: '.5s',
		equation: 'ease-out'
	},
	bind: {
		'sensor.click': 'press',
		'sensor.dblclick': '_onDblClick',
		'sensor.mousedown': '_onMouseDown',
		'sensor.mouseup': '_onMouseUp',
		'sensor.mouseleave': '_onMouseLeave',
		'sensor.mouseenter': '_onMouseEnter'
	}
};

/**
 * Button control class
 * @class
 * @extends Control
 * @since 0.0.1
 */
class Button extends Component {

	init(options) {
		super.init(options);

		this.name = 'button';

		this.options = [ defaults, options ].reduce(Object.assign, {});
	}
	/**
	 * [build description]
	 * @return {void}
	 */
	build(props) {
		super.build(props);

		var opts = this.options;
		var type = opts.type;

		opts.label = opts.label || opts.n;

		if (type === null) {
			type = 'icon-text';
		}

		if (opts.name) {
			this.element.attribute('data-name', opts.name);
		}

		this.element.attribute('title', opts.text);

		if (opts.icon) {
			this._initIcon(type, opts.icon || opts.name);
		}

		this._initLabel(type);

		if (opts.ink) {
			this._initSensor();
		} else {
			this.sensor = this.element;
		}
	}

	/**
	 * [_onElementMouseDown description]
	 * @param  {event} e
	 * @return {void}
	 */
	press(e) {
		e.stopPropagation();
		var opts = this.options;

		if (this.state === 'disabled') {
			return;
		}

		this._initInk(e);

		if (opts.emit) {
			this.emit(opts.emit);
		}

		this.emit('press', opts.emit);
		this.emit('pressed', opts.emit);

		return this;
	}

	/**
	 * [_initIcon description]
	 * @param  {string} type
	 * @return {string}
	 */
	_initIcon(type, name) {

		var tag = 'span';
		var code = name;

		var prop = {
			'class': 'ui-icon'
		};

		this.icon = new Element(tag, prop).inject(this.element);

		// prepare use of svg
		// this.iconsvg = new Element('svg', prop).inject(this.element);
		// this.svguse = new Element('use').inject(this.iconsvg);

		// this.iconsvg.attribute('viewBox', '0 0 24 24');
		// this.svguse.attribute('xlink:href','/vendor/icon/content-send.svg');

		if (code) {
			this.icon.addClass(code);
		}
	}

	/**
	 * [_initLabel description]
	 * @param  {string} type
	 * @return {void}
	 */
	_initLabel(type) {
		var options = this.options;

		var position = 'bottom';
		if (type === 'text-icon') {
			position = 'top';
		}

		if (this.options.label !== null) {
			var text = options.label || options.text;
			this.label = new Element('span.ui-text').inject(this.element, position);
			this.label.text(text);
		}
	}

	/**
	 * [_initText description]
	 * @return {void}
	 */
	_initSensor() {

		this.sensor = new Element('div.ui-sensor')
			.inject(this.element);
	}

	// INK

	/**
	 * [_onElementMouseDown description]
	 * @param  {event} e
	 * @return {Object}
	 */
	_initInk(e) {

		var element = this.element;

		var x = e.offsetX;
		var y = e.offsetY;

		var ink = this.ink = new Element('span.ui-ink').inject(element, 'top');

		this.ink.style({
			left: x - 5,
			top: y - 5
		});

		this._touchInk(ink);

		this.emit('mousedown');
	}

	/**
	 * [_initEffect description]
	 * @param  {string} ink
	 * @param  {string} x
	 * @param  {string} y
	 * @param  {Object} coord
	 * @return {void}
	 */
	_touchInk(ink) {

		var coord = this._inkCoord(ink);
		var options = this.options.ink;

		this.ink = ink;

		this.animate({
			width: coord.size+'px',
			height: coord.size+'px',
			left: '0',
			top: coord.top +'px',
			opacity: '0'
		}, options.duration, options.equation);

		var wait = options.duration.replace('s','')*1000;

		setTimeout(function() {
			ink.destroy();
		}, wait);
	}

	/**
	 * [_inkCoord description]
	 * @return {Object} Size and top
	 */
	_inkCoord() {
		var height = this.element.compute('height').replace('px', '');
		var width = this.element.compute('width').replace('px', '');

		width = parseInt(width);
		height = parseInt(height);

		var size = width;
		var top = 0;

		if (width > height) {
			size = width;
			top = (height - width) / 2;
		} else if (width < height) {
			size = height;
			top = (width - height) / 2;
		}

		return {
			size: size,
			top: top
		}
	}

	/**
	 * [animate description]
	 * @param  {Object} object   [description]
	 * @param  {interger} duration [description]
	 * @param  {string} equation [description]
	 */
	animate(object, duration, equation) {

		for (var key in object) {
			 if (object.hasOwnProperty(key)) {
			 	var prop = {};
			 	prop[key] = object[key];

				this.ink.animate(
					prop,
					{duration: duration},
					{equation: equation}
				);
			}
		}
	}

	// EVENTS

	/**
	 * [_onElementMouseDown description]
	 * @param  {event} e
	 * @return {void}
	 */
	_onDblClick(e) {
		var opts = this.options;

		e.stopPropagation();

		this.emit('dblpressed', opts.emit);
		return this;
	}

	/**
	 * [_onElementMouseUp description]
	 * @return {void}
	 */
	_onMouseDown(e) {

		return this;
	}

	/**
	 * [_onElementMouseUp description]
	 * @return {void}
	 */
	_onMouseUp(e) {

		return this;
	}

	/**
	 * [_onElementMouseUp description]
	 * @return {void}
	 */
	_onMouseLeave(e) {

		//console.log('mouseleave', e);

		return this;
	}

	/**
	 * [_onElementMouseDown description]
	 * @param  {event} e
	 * @return {void}
	 */
	_onMouseEnter(e) {

		this.emit('mouseenter', this.options.emit);

		return this;
	}

};

module.exports = Button;
